#include "screen.h"
#include "game_display.h"
#include "info.h"
#include "options.h"
#include "map.h"

#include <curses.h>
#include <stdlib.h>
#include <string.h>

#define IOBUF 80

struct xor_screen *screen_data = 0;

WINDOW *game_win = 0;

WINDOW *info_win = 0;

float info_w_ratio = MIN_INFO_W / (float) MIN_W;

su_t
screen_create()
{
    initscr();
    if (has_colors() == FALSE) {
        endwin();
        fprintf(stderr, "Your terminal does not support color\n");
        return 0;
    }
    noecho();
    nonl();
    cbreak();
    nodelay(game_win, FALSE);
    start_color();
    curs_set(0);
    if (screen_data)
        screen_destroy();
    screen_data = malloc(sizeof(struct xor_screen));
    if (!screen_data)
        return 0;
    init_pair(ICON_XXX + 1, COLOR_BLACK, COLOR_YELLOW);
    return 1;
}

void
screen_destroy()
{
    if (screen_data) {
        free(screen_data);
        screen_data = 0;
    }
    if (game_win) {
        delwin(game_win);
        game_win = 0;
    }
    if (info_win) {
        delwin(info_win);
        info_win = 0;
    }
    endwin();
    printf("Goodbye!\n");
}

su_t
screen_size()
{
    short maxx = getmaxx(stdscr);
    short maxy = getmaxy(stdscr);
    if (maxx < MIN_W || maxy < 24)
        return 0;
    /*  The info panel width is calculated from a ratio and then
       restricted to just large enough to accommodate a map display. */
    screen_data->info_w = info_w_ratio * maxx;
    if (screen_data->info_w >= MAP_W + 2)
        screen_data->info_w = MAP_W + 2;

    /*  game area width */
    screen_data->garea_w = (options->oldschool_play
                            ? DEF_GAREA_W
                            : (maxx - screen_data->info_w) / ICON_W);
    /* game area top-left x coordinate */
    screen_data->ga_tlx =
        (maxx - screen_data->info_w - screen_data->garea_w * ICON_W) / 2;
    /* centering... */
    if (screen_data->garea_w > MAX_GAREA_W) {
        screen_data->ga_tlx +=
            ((screen_data->garea_w - MAX_GAREA_W) * ICON_W) / 2;
        screen_data->garea_w = MAX_GAREA_W;
    }
    /* game area height... */
    screen_data->garea_h = (options->oldschool_play
                            ? DEF_GAREA_H : maxy / ICON_H);
    /* game area top-left y coordinate */
    screen_data->ga_tly = (maxy - screen_data->garea_h * ICON_H) / 2;
    /* centering... */
    if (screen_data->garea_h > MAX_GAREA_H) {
        screen_data->ga_tly +=
            ((screen_data->garea_h - MAX_GAREA_H) * ICON_H) / 2;
        screen_data->garea_h = MAX_GAREA_H;
    }
    /* info window height */

    screen_data->info_h =
        (maxy > MAX_INFO_H_OLDSCHOOL ? MAX_INFO_H_OLDSCHOOL : maxy);

/*  this commented out because it only made sense when the largest
    possible game area was large enough to accommodate a full detail
    map display (heightwise).
    if (options->oldschool_play)
        screen_data->info_h = (maxy > MAX_INFO_H_OLDSCHOOL ? MAX_INFO_H_OLDSCHOOL : maxy);
    else
        screen_data->info_h = (maxy > MAX_INFO_H ? MAX_INFO_H : maxy);
*/
    /* info window too small for unscaled map display ? */
    if (screen_data->info_h - 6 <= MAP_H || screen_data->info_w - 2 < MAP_W) {
        screen_data->scale_map = TRUE;  /* scale it down that is */
        screen_data->info_h = screen_data->garea_h * ICON_H;
        screen_data->i_tly = screen_data->ga_tly;
        screen_data->map_tlx = 1 + (screen_data->info_w - 2 - HMAP_W) / 2;
        screen_data->map_tly = 5 + (screen_data->info_h - 6 - HMAP_H) / 2;
    }
    else {
        screen_data->scale_map = FALSE;
        screen_data->i_tly = (maxy - screen_data->info_h) / 2;
        screen_data->map_tlx = 1 + (screen_data->info_w - 2 - MAP_W) / 2;
        screen_data->map_tly = 6 + (screen_data->info_h - 6 - MAP_H) / 2;
    }
    screen_data->i_tlx = screen_data->ga_tlx + screen_data->garea_w * ICON_W;
    if (game_win)
        delwin(game_win);
    game_win = newwin(screen_data->garea_h * ICON_H,
                      screen_data->garea_w * ICON_W,
                      screen_data->ga_tly, screen_data->ga_tlx);
    if (!game_win)
        return 0;
    scrollok(game_win, FALSE);
    nodelay(game_win, FALSE);
    keypad(game_win, TRUE);
    if (info_win)
        delwin(info_win);
    info_win = newwin(screen_data->info_h,
                      screen_data->info_w,
                      screen_data->i_tly, screen_data->i_tlx);
    if (!info_win)
        return 0;
    scrollok(info_win, FALSE);
    keypad(info_win, TRUE);
    return 1;
}

su_t
screen_resize()
{
    if (!screen_size()) {
        char warn[] = "** TOO SMALL **";
        char wc[2] = " ";
        su_t wl = strlen(warn);
        do {
            short maxx = getmaxx(stdscr);
            short maxy = getmaxy(stdscr);
            attrset(COLOR_PAIR(ICON_XXX + 1));
            ctr_t c = 0;
            for (short y = 0; y < maxy; y++) {
                for (short x = 0; x < maxx; x++) {
                    wc[0] = (c < wl ? warn[c] : '/');
                    mvwprintw(stdscr, y, x, wc);
                    c++;
                    if (c == 37)
                        c = 0;
                }
            }
            wgetch(stdscr);
        } while (!screen_size());
    }
    werase(stdscr);
    wrefresh(stdscr);
    if (screen_data->game_win_repaint_cb)
        screen_data->game_win_repaint_cb();
    if (screen_data->info_win_repaint_cb)
        screen_data->info_win_repaint_cb();
    return 1;
}

int
scr_wmsg(WINDOW * win, char *msg, short len, struct xy *msgxy)
{
    if (!win || !msg)
        return 0;
    if (!len)
        len = strlen(msg);
    wattrset(win, COLOR_PAIR(ICON_XXX + 1));
    bool autofree = FALSE;
    if (!msgxy) {
        msgxy = malloc(sizeof(struct xy));
        autofree = TRUE;
    }
    msgxy->x = (getmaxx(win) - (len + 2)) / 2;
    msgxy->y = (getmaxy(win) - 3) / 2;
    char spc[IOBUF + 1];
    for (int i = 0; i <= IOBUF; ++i)
        spc[i] = ' ';
    if (len > IOBUF)
        len = IOBUF;
    spc[len] = '\0';
    for (int yy = msgxy->y - 1; yy <= msgxy->y + 1; yy++)
        mvwprintw(win, yy, msgxy->x, spc);
    mvwprintw(win, msgxy->y, msgxy->x, msg);
    if (autofree)
        free(msgxy);
    return len;
}

char *
scr_wmsg_read(WINDOW * win, char *msg, short readlen)
{
    if (!win || !msg)
        return 0;
    short len = strlen(msg);
    short maxx = getmaxx(win);
    short mxl = (IOBUF > maxx ? maxx : IOBUF);
    if (len >= mxl)
        return 0;
    if (!readlen)
        readlen = mxl - len;
    if (readlen < 0)
        return 0;
    short tlen = len + readlen;
    if (tlen > mxl)
        return 0;
    wattrset(win, COLOR_PAIR(ICON_XXX + 1));
    struct xy msgxy;
    msgxy.x = (maxx - (tlen + 2)) / 2;
    msgxy.y = (getmaxy(win) - 3) / 2;
    char spc[mxl];
    for (int i = 0; i <= mxl; ++i)
        spc[i] = ' ';
    spc[tlen] = '\0';
    for (int yy = msgxy.y - 1; yy <= msgxy.y + 1; yy++)
        mvwprintw(win, yy, msgxy.x, spc);
    mvwprintw(win, msgxy.y, msgxy.x, msg);
    char buf[mxl];
    wmove(game_win, msgxy.y, msgxy.x + len);
    echo();
    wgetnstr(game_win, buf, readlen);
    noecho();
    char *ret = malloc(strlen(buf) + 1);
    strcpy(ret, buf);
    return ret;
}
