/****************************************************************************
    This file is part of XorCurses a port/remake of the game Xor
    (originally by Astral Software) to the Linux console using
    ncurses.

    XorCurses written by James W. Morris
    http://www.jwm-art.net/

    All code licensed under GNU GPL.

    file:       screen.h
    purpose:    handles screen (terminal) sizing and resizing, allocates
                windows for the game area and info area.

****************************************************************************/
#ifndef SCREEN_H
#define SCREEN_H

#include <curses.h>

#include "types.h"
#include "icons.h"

#define MIN_W       77      /*  minimum width of terminal.          */

#define DEF_GAREA_W 8       /*  for oldschool purists.              */
#define DEF_GAREA_H 8       

#define MAX_GAREA_W 12      /*  maximum width of game area.         */
#define MAX_GAREA_H 12      /*  maximum width of game area.         */

#define MIN_INFO_W  32      /*  width of the info window.           */

/*  info window height control settings for oldschool and newschool
    difficulty levels:                                              */
#define MAX_INFO_H MAX_GAREA_H * ICON_H
#define MAX_INFO_H_OLDSCHOOL 40

extern WINDOW *info_win;

extern WINDOW *game_win;

struct xor_screen
{
    bool ga_resized;/*  TRUE if game area resized.                  */
    xy_t ga_tlx;    /*  game area top left x and y.                 */
    xy_t ga_tly;
    xy_t garea_w;   /*  count of icons width-wise in game area.     */
    xy_t garea_h;   /*  count of icons height-wise in game area.    */

    xy_t i_tlx;     /*  info window top left x and y                */
    xy_t i_tly;
    xy_t info_w;    /*  width of info win (chars).                  */
    xy_t info_h;    /*  height of info win (chars).                 */

    xy_t map_tlx;   /*  top left x and y of map display within the  */
    xy_t map_tly;   /*  info window.                                */

    bool scale_map; /*  TRUE when screen is too small to display    */
                    /*  both game area and full-size map.           */

    /*  callbacks for repainting called by screen_resize            */
    void (*game_win_repaint_cb)();
    void (*info_win_repaint_cb)();
};

extern struct xor_screen *screen_data;

su_t screen_create();   /* init curses and create the screen_data   */

void screen_destroy();

/*  screen_size calculates geometries of game area and info window.
    it is only ever called by screen_resize, with the exception of
    that screen_size should be called once after ncurses initialises.
    returns 0 if size too small.
    returns 1 otherwise.
*/
su_t screen_size();

/*  screen_resize() should be called when the terminal is resized -
    either when KEY_RESIZE is returned by getch, or something changes
    which effects the geometries of the game area and info window
    (changing the difficulty level does this is one example).
*/
su_t screen_resize();

int scr_wmsg(WINDOW *win, char *msg, short len, struct xy *msgxy);

char* scr_wmsg_read(WINDOW *win, char *msg, short readlen);

#endif

